// Keys.hh for Fluxbox 
// Copyright (c) 2001 Henrik Kinnunen (fluxgen@linuxmail.org)
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.	IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.

//  bbkeys.hh for bbkeys - Ye Olde Keystroke Grabber for Blackbox
//
//	Copyright (c) 1998-1999 by John Kennis, jkennis@chello.nl
//	Copyright (c) 1999-2001 by Jason Kasper (vanRijn) vR@movingparts.net
//	Copyright (c) 2001 by Ben Jansens <xor@x-o-r.net>
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#ifndef __BBKEYS_HH
#define __BBKEYS_HH

#include "grab_defs.hh" // grab* definitions

#include <X11/Xlib.h>
#include <X11/keysym.h>

#include <string>

#define MaxInstructions 50

typedef struct {
		unsigned long modMask;
		KeyCode keycode;
		int action;
		char *execCommand;
} KEY_ACTION_MAP;

typedef struct {
		int instructCount;
		KEY_ACTION_MAP KeyMap[MaxInstructions];
} KEY_GRAB_INSTRUCTION;

struct Action_t {
	const char *action;
	int grabaction;
};
	
class Keys {
public:
	Keys(const char *);
	~Keys(void);

	void CheckConfig(const char *rcfile);
	int getAction(XKeyPressedEvent *ke);
	const char *getActionString(int action);
	const char *getExecCommand();
private:

	// functions for translation of grab information
	char *index_to_name(int);
	int translateAction(char *);
	int translateModifier(char *);

	// functions for handling/grabbing keys
	void loadKeygrabs(const char *);
	void activateKeygrabs(void);
	void getOffendingModifiers(void);
	void wHackedGrabKey(int,unsigned int,Window,Bool,int,int);
	void x_reset_modifier_mapping(Display *);
	void InitializeModifiers(void);

	unsigned int KeycodeToModmask(unsigned int code);
	
	static const Action_t actions[];	//action table
	
	KEY_GRAB_INSTRUCTION grabSet; // key grab/handling vars
	int actionList[NUM_GRABS];	//list of actions, load from file
	unsigned int ValidModMask;
	unsigned int _NumLockMask;
	unsigned int _ScrollLockMask;
	//execute command string
	string exec_command;
};

#endif /* __BBKEYS_HH */
